# -*- coding: utf-8 -*-
"""

Analyzing convergence of GrIS ensemble statistics as
a function of number of members
v1

"""


import os
import numpy as np
import netCDF4 as nc
import matplotlib.pyplot as plt
from scipy import stats


nruns           = 200 #nb of ensemble members
nsteps          = 2000 #nb of time steps
nmembsplot      = np.arange(5,201,5)

DirResults   = os.path.dirname(os.path.realpath(__file__))+'/'
resPath      = DirResults+'NCfile_StochClimGrIS_200Ens_v1.nc'

mycol0 = 'dodgerblue'

ds = nc.Dataset(resPath)
### Load ###
timear         = np.array(ds.variables['time'])
fullmatMvals       = 917*1e-12*np.array(ds.variables['volume'])[0:nruns,:] #[Gt]
ds.close()
### Stats on the full ensemble ###
MnFullMvals         = np.mean(fullmatMvals,axis=0)
SdevFullMvals       = np.std(fullmatMvals,axis=0)
SkewFullMvals       = stats.skew(fullmatMvals,axis=0)

### Stats for random selection of ensemble members ###
MnSelMvals      = np.zeros((len(nmembsplot),len(timear)))
SdevSelMvals    = np.zeros((len(nmembsplot),len(timear)))
SkewSelMvals    = np.zeros((len(nmembsplot),len(timear)))

isel = []
for nn,num in enumerate(nmembsplot):
    if(nn==0):
        isel.append(np.random.randint(0,nruns,size=num))
    else:
        allinds  = np.random.randint(0,nruns,nruns).tolist() #all indices in random order
        numextra = num-nmembsplot[nn-1]
        indsnew  = [ind for ind in allinds if((ind in isel[nn-1])==False)][0:numextra]
        isel.append(np.append(isel[nn-1],indsnew))

for kk,indices in enumerate(isel):
    selmatMvals          = fullmatMvals[indices,:]
    MnSelMvals[kk,:]     = np.mean(selmatMvals,axis=0)
    SdevSelMvals[kk,:]   = np.std(selmatMvals,axis=0)
    SkewSelMvals[kk,:]   = stats.skew(selmatMvals,axis=0)

### Take relative values to the full ensemble values ###
relMnSelMvals    = MnSelMvals/MnFullMvals
relSdevSelMvals  = SdevSelMvals/SdevFullMvals

### Plotting ###
fig = plt.figure(figsize=[11,10])
ax  = plt.subplot(311)
ax.plot(nmembsplot,1e-6*MnSelMvals[:,-1],c=mycol0,linewidth=1.0,marker='o',markersize=5)
ax.tick_params(axis='y', which='major', labelsize=14)
ax.set_ylabel(r'Mean final ice mass [$\times 10^6$ Gt]',fontsize=14)
ax.tick_params(axis='y', which='major',labelsize=14)
ax.tick_params(axis='x', which='major',labelcolor='w')
ax.text(-2,np.max(1e-6*MnSelMvals[:,-1]-0.0002),'(a)',fontsize=14)
ax  = plt.subplot(312)
ax.plot(nmembsplot,relSdevSelMvals[:,-1],c=mycol0,linewidth=1.0,marker='o',markersize=5)
ax.tick_params(axis='y', which='major', labelsize=14)
ax.set_ylabel(r'$\dfrac{\sigma_{E}}{\sigma_{E,200}}$ in final ice mass',fontsize=14)
ax.tick_params(axis='y', which='major',labelsize=14)
ax.tick_params(axis='x', which='major',labelcolor='w')
ax.text(-2,np.max(relSdevSelMvals[:,-1]-0.02),'(b)',fontsize=14)
ax  = plt.subplot(313)
ax.plot(nmembsplot,SkewSelMvals[:,-1],c=mycol0,linewidth=1.0,marker='o',markersize=5)
ax.tick_params(axis='y', which='major', labelsize=14)
ax.set_ylabel('Skew in final ice mass',fontsize=14)
ax.tick_params(axis='y', which='major',labelsize=14)
ax.tick_params(axis='x', which='major',labelsize=14)
ax.text(-2,np.max(SkewSelMvals[:,-1]-0.03),'(c)',fontsize=14)
ax.set_xlabel('Number of ensemble members',fontsize=14)
fig.tight_layout()
